#require "hbmxml.hbc"

#include "hmg.ch"

Function Main
Local cXmlString := FileStr( "test.xml" )
Local hHash      := hb_xmlToHash( cXmlString, .T. /* .F. = encode root node also */  )
Local cJsonString:= hb_jsonEncode ( hHash, .T. )
StrFile ( cJsonString, "test.json" )

cJsonString := FileStr( "test.json" )
hHash       := hb_jsonDecode ( cJsonString )
cXmlString  := hb_hashToXML ( hHash, 0 )
StrFile ( cXmlString, "test2.xml" )


Return

********************************************************
*     Conversion  hb_hashToXML() / hb_XMLtoHash()      *
*              code by Przemyslaw Czerpak              *
********************************************************

FUNCTION hb_hashToXML( hValue, n, cEOL )
   LOCAL cXML
   IF valtype( n ) != "N"
      n := -2
   ENDIF
   IF cEOL == NIL
      cEOL := iif( n >= -1, hb_eol(), "" )
   ENDIF
   cXML := hb_XML_savenodes( hValue, n, cEOL )
   RETURN '<?xml version="1.0" encoding="UTF-8"?>' + hb_eol() + hb_strToUtf8( cXML ) + hb_eol()


STATIC FUNCTION hb_XML_savenodes( hValue, n, cEOL )
   LOCAL cXML := "", v, e, x, n2
   n2 := iif( n < 0, n, n + 2 )
   for each v in hValue
      SWITCH valtype( v )
      CASE "C"
      CASE "M"
         x := v
         EXIT
      CASE "N"
         x := hb_ntos( v )
         EXIT
      CASE "D"
         x := dtos( v )
         EXIT
      CASE "T"
         x := hb_tstostr( v )
         EXIT
      CASE "L"
         x := iif( v, "true", "false" )
         EXIT
      CASE "H"
         cXML += space( n ) + "<"  + v:__enumKey() + ">" + cEOL
         cXML += hb_XML_savenodes( v, n2, cEOL )
         cXML += space( n ) + "</" + hb_XML_endtag( v:__enumKey() ) + ">" + cEOL
         x := NIL
         EXIT
      CASE "A"
         FOR EACH e IN v
            cXML += space( n ) + "<"  + v:__enumKey() + ">" + cEOL
            cXML += hb_XML_savenodes( e, n2, cEOL )
            cXML += space( n ) + "</" + hb_XML_endtag( v:__enumKey() ) + ">" + cEOL
         next
         x := NIL
         EXIT
      OTHERWISE
         x := NIL
      ENDSWITCH
      IF x != NIL
         cXML += space( n ) + "<" + v:__enumKey() + ">"
         cXML += x
         cXML += "</" + hb_XML_endtag( v:__enumKey() ) + ">" + cEOL
      ENDIF
   next
   RETURN cXML


STATIC FUNCTION hb_XML_endtag( cTag )
   LOCAL n := at( " ", cTag )
   IF n != 0
      cTag := left( cTag, n - 1 )
   ENDIF
   RETURN cTag


FUNCTION hb_XMLtoHash( cXML, lOmitHeader )
   LOCAL pXML

   hb_default( @lOmitHeader, .T. )

   pXML := mxmlLoadString( NIL, hb_utf8ToStr( cXML ), MXML_OPAQUE_CALLBACK )
   IF !empty( pXML )
      IF lOmitHeader
         RETURN hb_XML_getnodes( mxmlGetFirstChild( pXML ) )
      ELSE
         RETURN hb_XML_getnodes( pXML )
      ENDIF
   ENDIF
   RETURN NIL


STATIC FUNCTION hb_XML_getnodes( pNode )
   LOCAL cKey, pChild, xResult, xValue, aValue

   WHILE  !empty( pNode )
      IF mxmlGetType( pNode ) == MXML_ELEMENT
         cKey := mxmlGetElement( pNode )
         pChild := mxmlGetFirstChild( pNode )
         xValue := hb_XML_getnodes( pChild )
         IF xValue == NIL
            xValue := mxmlGetOpaque( pNode )
         ENDIF
         IF xResult == NIL
            xResult := {=>}
            hb_hKeepOrder( xResult, .t. )
            xResult[ cKey ] := xValue
         ELSEIF cKey $ xResult
            aValue := xResult[ cKey ]
            IF !hb_isArray( aValue )
               aValue := { aValue }
               xResult[ cKey ] := aValue
            ENDIF
            aadd( aValue, xValue )
         ELSE
            xResult[ cKey ] := xValue
         ENDIF
      ENDIF
      pNode := mxmlGetNextSibling( pNode )
   ENDDO
   RETURN xResult


